#pragma once

#include "CuckooHashTable.h"

#include "../Pair.hpp"


template<typename K, typename V, typename Device = TNL::Devices::Host>
class CuckooHashMap : public CuckooHashTable<Pair<K, V>, K, Device> {
  public:
    ///< Copy constructor
    CuckooHashMap(const CuckooHashMap<K, V, Device>& other);

    /**Constructor
     * values - vector of <key, value> pairs to populate the table
     * nHashFunctions = number of hash functions to generate and use
     */
    CuckooHashMap(const Array<Pair<K, V>, Device>& values = {}, uint8_t nHashFunctions = 6);

    /**Constructor
     * 
     * tableSize - number of rows in the table
     * nHashFunctions - number of hash functions to generate and use
     * nIters - maximal number of insertion attempts before starting over with regenerated hash functions
     * values - vector of <key, value> pairs to populate the table
     */
    CuckooHashMap(int tableSize, uint8_t nHashFunctions, int nIters, 
                  const std::vector<std::pair<K, V>>& values = {});

    /**Find a given key in the table if it exists
     * If the key is found, return true and write the related value into 'value' out-parameter
     */
    bool find(const K& key, V& value) const;
};


#include "CuckooHashMap.hpp"
