#pragma once

#include "CuckooHashTable.h"

#include "../Value.hpp"

#include <TNL/Containers/Array.hpp>


using namespace TNL::Containers;

template<typename T, typename Device = TNL::Devices::Host>
class CuckooHashSet : public CuckooHashTable<Value<T>, T, Device> {
  public:
    ///< Copy constructor
    CuckooHashSet(const CuckooHashSet<T, Device>& other);

    ///< Move constructor
    CuckooHashSet(CuckooHashSet<T, Device>&& other);

    /**Constructor
     * values - vector of <key, value> pairs to populate the table
     * nHashFunctions = number of hash functions to generate and use
     */
    CuckooHashSet(const Array<T, Device>& values = {}, uint8_t nHashFunctions = 6);

    /**Constructor
     * 
     * tableSize - number of rows in the table
     * nHashFunctions - number of hash functions to generate and use
     * nIters - maximal number of insertion attempts before starting over with regenerated hash functions
     * values - vector of values to populate the table
     */
    CuckooHashSet(int tableSize, uint8_t nHashFunctions, int nIters, 
                    const Array<T, Device>& values = {});

    ///< Check if given key exists in the table
    bool contains(const T& value) const;
};


#include "CuckooHashSet.hpp"
