#pragma once

#include "CuckooHashTable.h"

#include <TNL/Containers/StaticArray.h>


template<typename Item, typename Key, typename Device>
CuckooHashTable<Item, Key, Device>::CuckooHashTable(const CuckooHashTable<Item, Key, Device>& other) :
        m_iterations(other.m_iterations),
        m_hashFunctions(other.m_hashFunctions),
        m_table(other.m_table),
        m_content(other.m_content),
        m_view(other.m_view) {}

template<typename Item, typename Key, typename Device>
CuckooHashTable<Item, Key, Device>::CuckooHashTable(const Array<Item, Device>& items, uint8_t nHashFunctions) : 
    CuckooHashTable<Item, Key, Device>(1.25 * items.getSize(), nHashFunctions, 7 * log(items.getSize()), items) {}

template<typename Item, typename Key, typename Device>
CuckooHashTable<Item, Key, Device>::CuckooHashTable(int tableSize, uint8_t nHashFunctions, int nIters,
                                                    const Array<Item, Device>& items) :
        m_iterations(nIters),
        m_hashFunctions(nHashFunctions),
        m_table(tableSize),
        m_content(items.getSize()),
        m_view(std::make_shared<ViewType>(*this, items.getConstView())) {} 

template<typename Item, typename Key, typename Device>
CuckooHashTable<Item, Key, Device>::CuckooHashTable(CuckooHashTable<Item, Key, Device>&& other) :
        m_iterations(other.m_iterations),
        m_hashFunctions(std::move(other.m_hashFunctions)),
        m_table(std::move(other.m_table)),
        m_content(std::move(other.m_content)),
        m_view(std::move(other.m_view)) {}

template<typename Item, typename Key, typename Device>
int CuckooHashTable<Item, Key, Device>::generations() const { return m_view->generations(); }

template<typename Item, typename Key, typename Device>
int CuckooHashTable<Item, Key, Device>::duplicates() const { return m_view->duplicates(); }
