#pragma once

#include "CuckooHashTable.h"

#include "../HashFunction.h"
#include "../RandomIntGenerator.h"

#include <TNL/Containers/Array.h>


using namespace TNL::Containers;

template<typename Item, typename Key, typename Device>
class CuckooHashTable;

/** CuckooHashTable - HashMap for template types
 * 
 * K - type of keys
 * V - type of values
 */
template<typename Item, typename Key, typename Device>
class CuckooHashTableView {
    using TableType = CuckooHashTable<Item, Key, Device>;

  public:
    struct Entry {
      Item item;
      int hashFunction;
    };

    CuckooHashTableView(TableType& table, const typename Array<Item, Device>::ConstViewType items);
  
    //! Find and return item having provided key
    bool find(const Key& key, ArrayView<Item, Device> item) const;

    ///< Reports the number of performed attempts to build the table
    int generations() const;

    ///< Reports the number of attempts to insert duplicates while building the table
    int duplicates() const;

    //! Randomly generate hash functions
    void init_hash_functions();

    /**Inserts a new item into the table
     * Returns true if insertion is successful (didn't reach maximum number of iterations)
     */
    bool insert(Item item, bool& duplicate);

    //! Populate the table with given values, regenerate hash functions on each failure
    void build();
    
    void copy_items(const typename Array<Item, Device>::ConstViewType items);

  protected:
    ArrayView<HashFunction<Key>, Device> m_hashFunctions;
    ArrayView<int, Device> m_table;
    ArrayView<Entry, Device> m_content;

  private:
    int m_iterations;
    // friend bool insert_impl(Item item, bool& duplicate, 
    //                         decltype(m_iterations) iter,
    //                         decltype(m_hashFunctions) hf, 
    //                         decltype(m_table) table);

    RandomIntGenerator m_rand;
    int m_generations;
    int m_duplicates;
};

#include "CuckooHashTableView.hpp"
