#pragma once

#include "HashFunction.h"

#include "Triangle.hpp"

#include <TNL/Containers/StaticArray.h>

#include <climits>
#include <string>
#include <hash_fun.h>


template<typename T>
HashFunction<T>::HashFunction(int a, int b, int max) : a(a), b(b), max(max) {}

template<typename T>
HashFunction<T>::HashFunction(RandomIntGenerator& rand, int max) : a(abs(rand())), b(abs(rand())), max(max) {}

template<typename T>
int HashFunction<T>::operator()(const T& value) const {
    return (a * hash(value) + b) % PRIME % max % __INT_MAX__;
}

template<typename T>
void HashFunction<T>::init(RandomIntGenerator& rand, int max) {
    a = abs(rand());
    b = abs(rand());
    this->max = max;
}


//////////////// SPECIALIZATIONS ////////////////
// std::string
// template<>
// size_t HashFunction<std::string>::hash(const std::string& value) const {
//     static std::hash<std::string> hashFun;
//     return hashFun(value);
// }

// //template class HashFunction<std::string>;


// TNL::Containers::StaticArray<int, 3>
using Coordinates3D = TNL::Containers::StaticArray<3, int>;

template<>
size_t HashFunction<Coordinates3D>::hash(const Coordinates3D& value) const {
    int hashValue = PRIME;
    for (int i = 0; i < 3; i++)
        hashValue = hashValue * 19 + value[i];
    return abs(hashValue);
}

//template class HashFunction<Coordinates3D>;


// TNL::Containers::StaticArray<4, int>
using Cell = TNL::Containers::StaticArray<4, int>;

template<>
size_t HashFunction<Cell>::hash(const Cell& value) const {
    int hashValue = PRIME;
    for (int i = 0; i < 4; i++)
        hashValue = hashValue * 19 + value[i];
    return abs(hashValue);
}

//template class HashFunction<Cell>;


// Triangle
template<>
size_t HashFunction<TestSet::Triangle>::hash(const TestSet::Triangle& value) const {
    int hashValue = PRIME;
    for (int i = 0; i < 3; i++)
        hashValue = hashValue * 19 + value.v[i];
    return abs(hashValue);
}

//template class HashFunction<TestSet::Triangle>;

// Numerical
template<>
size_t HashFunction<int>::hash(const int& value) const {
    return size_t(value);
}

//template class HashFunction<int>;

template<>
size_t HashFunction<long>::hash(const long& value) const {
    return size_t(value);
}

//template class HashFunction<long>;

template<>
size_t HashFunction<float>::hash(const float& value) const {
    return size_t(value);
}

//template class HashFunction<float>;

template<>
size_t HashFunction<double>::hash(const double& value) const {
    return size_t(value);
}

//template class HashFunction<double>;
