#pragma once

#include "HashGraphS.h"

#include "../../HashFunction.h"

#include <TNL/Algorithms/Segments/CSRView.h>
#include <TNL/Containers/ArrayView.h>
#include <TNL/Containers/VectorView.h>

#include <vector>


using namespace TNL::Algorithms::Segments;
using namespace TNL::Containers;

template<typename Item, typename Key, typename Device>
class HashGraphS;

template<typename Item, typename Key, typename Device>
class HashGraphSView {
    using TableType = HashGraphS<Item, Key, Device>;

  public:
    HashGraphSView(TableType& table,
                   const typename Array<Item, Device>::ConstViewType items);
    ~HashGraphSView();
    int duplicates() const;

    bool find(const Key& key, ArrayView<Item, Device> item) const;

    void build();

    void fill_csr();

  private:
    ArrayView<Item, Device> m_content;
    Array<Item, Device> m_table;
    CSRView<Device, int> m_csr;
    VectorView<int, Device> m_segmentsSizes;
    HashFunction<Key> m_hash;
};


#include "HashGraphSView.hpp"
