#pragma once

#include "../CuckooHash/CuckooHashSet.h"

#include "gtest/gtest.h"

#include <string>
#include <type_traits>
#include <vector>


#ifdef HAVE_CUDA
#include <TNL/Devices/Cuda.h>
using Device = TNL::Devices::Cuda;
#else
#include <TNL/Devices/Host.h>
using Device = TNL::Devices::Host;
#endif


template<typename Key>
class CuckooHashSetTest : public ::testing::Test {
protected:
    using KeyType = Key;
    using ArrayType = Array<KeyType, Device>;
    using SetType = CuckooHashSet<Key, Device>;
};

using CuckooHashSetTypes = ::testing::Types<int, long, float, double>;

TYPED_TEST_SUITE(CuckooHashSetTest, CuckooHashSetTypes);

TYPED_TEST(CuckooHashSetTest, correctQuery) {
    using KeyType = typename TestFixture::KeyType;
    using ArrayType = typename TestFixture::ArrayType;
    using SetType = typename TestFixture::SetType;

    ArrayType values = get_values<KeyType, Device>();
    int tableSize = 1.25 * values.getSize();
    int nIterations = 7 * log(values.getSize()); 
    SetType set(values, 6);

    for (int i = 0; i < values.getSize(); i++)
        EXPECT_TRUE(set.contains(values.getElement(i)));
}

TYPED_TEST(CuckooHashSetTest, wrongQuery) {
    using KeyType = typename TestFixture::KeyType;
    using ArrayType = typename TestFixture::ArrayType;
    using SetType = typename TestFixture::SetType;

    ArrayType values = get_values<KeyType, Device>();
    int tableSize = 1.25 * values.getSize();
    int nIterations = 7 * log(values.getSize()); 
    SetType map(values, 6);

    for (KeyType key = 8; key < 16; key++)
        EXPECT_FALSE(map.contains(key));
}
