#pragma once

#include "../HashGraph/HashGraphMap.h"

#include "gtest/gtest.h"

#include <string>
#include <type_traits>
#include <vector>


#ifdef HAVE_CUDA
#include <TNL/Devices/Cuda.h>
using Device = TNL::Devices::Cuda;
#else
#include <TNL/Devices/Host.h>
using Device = TNL::Devices::Host;
#endif


template<typename tPair>
class HashGraphV2MapTest : public ::testing::Test {
protected:
    using KeyType = typename std::tuple_element<0, tPair>::type;
    using ValueType = typename std::tuple_element<1, tPair>::type;
    using ArrayType = Array<Pair<KeyType, ValueType>, Device>;
    using MapType = HashGraphV2Map<KeyType, ValueType, Device>;
};

using HashGraphV2MapTypes = ::testing::Types<
    std::pair<int, int>,
    std::pair<int, long>,
    std::pair<int, float>,
    std::pair<int, double>,
    std::pair<long, int>,
    std::pair<long, long>,
    std::pair<long, float>,
    std::pair<long, double>,
    std::pair<float, int>,
    std::pair<float, long>,
    std::pair<float, float>,
    std::pair<float, double>,
    std::pair<double, int>,
    std::pair<double, long>,
    std::pair<double, float>,
    std::pair<double, double>
>;

TYPED_TEST_SUITE(HashGraphV2MapTest, HashGraphV2MapTypes);

TYPED_TEST(HashGraphV2MapTest, correctQuery) {
    using KeyType = typename TestFixture::KeyType;
    using ValueType = typename TestFixture::ValueType;
    using ArrayType = typename TestFixture::ArrayType;
    using MapType = typename TestFixture::MapType;

    ArrayType pairs = get_pairs<KeyType, ValueType, Device>();
    MapType map(pairs);
    ValueType value;
    for (int i = 0; i < pairs.getSize(); i++) {
        auto pair = pairs.getElement(i);
        KeyType key = pair.key;
        EXPECT_TRUE(map.find(key, value));
        EXPECT_EQ(value, pair.value);
    }
}

TYPED_TEST(HashGraphV2MapTest, wrongQuery) {
    using KeyType = typename TestFixture::KeyType;
    using ValueType = typename TestFixture::ValueType;
    using ArrayType = typename TestFixture::ArrayType;
    using MapType = typename TestFixture::MapType;

    ArrayType pairs = get_pairs<KeyType, ValueType, Device>();
    MapType map(pairs);

    for (KeyType key = 8; key < 16; key++) {
        ValueType value;
        EXPECT_FALSE(map.find(key, value));
    }
}
