#pragma once

#include "../HashGraph/HashGraphSet.h"

#include "gtest/gtest.h"

#include <string>
#include <type_traits>
#include <vector>


#ifdef HAVE_CUDA
#include <TNL/Devices/Cuda.h>
using Device = TNL::Devices::Cuda;
#else
#include <TNL/Devices/Host.h>
using Device = TNL::Devices::Host;
#endif


template<typename Key>
class HashGraphV2SetTest : public ::testing::Test {
protected:
    using KeyType = Key;
    using ArrayType = Array<KeyType, Device>;
    using SetType = HashGraphV2Set<KeyType, Device>;
};

using HashGraphSetTypes = ::testing::Types<int, long, float, double>;

TYPED_TEST_SUITE(HashGraphV2SetTest, HashGraphSetTypes);

TYPED_TEST(HashGraphV2SetTest, correctQuery) {
    using KeyType = typename TestFixture::KeyType;
    using ArrayType = typename TestFixture::ArrayType;
    using SetType = typename TestFixture::SetType;

    ArrayType values = get_values<KeyType, Device>();
    SetType set(values);

    for (int i = 0; i < values.getSize(); i++)
        EXPECT_TRUE(set.contains(values.getElement(i)));
}

TYPED_TEST(HashGraphV2SetTest, wrongQuery) {
    using KeyType = typename TestFixture::KeyType;
    using ArrayType = typename TestFixture::ArrayType;
    using SetType = typename TestFixture::SetType;

    ArrayType values = get_values<KeyType, Device>();
    SetType set(values);

    for (KeyType key = 8; key < 16; key++)
        EXPECT_FALSE(set.contains(key));
}
