#include "../Pair.hpp"

#include <TNL/Algorithms/ParallelFor.h>
#include <TNL/Containers/Array.h>

using namespace TNL::Containers;


template<typename KeyType, typename ValueType, typename Device>
Array<Pair<KeyType, ValueType>, Device> get_pairs() {
    Array<Pair<KeyType, ValueType>, Device> pairs(8);
    auto pView = pairs.getView();
    auto fill_pairs = [pView] __cuda_callable__ (int i) mutable {
        pView[i] = { KeyType(i), ValueType(i + 1) };
    };
    TNL::Algorithms::ParallelFor<Device>::exec(0, 8, fill_pairs);
    return pairs;
}

#include "CuckooHashMapTest.h"
#include "HashGraphV1MapTest.h"
#include "HashGraphV2MapTest.h"

template<typename KeyType, typename Device>
Array<KeyType, Device> get_values() {
    Array<KeyType, Device> values(8);
    auto vView = values.getView();
    auto fill_values = [vView] __cuda_callable__ (int i) mutable {
        vView[i] = KeyType(i);
    };
    TNL::Algorithms::ParallelFor<Device>::exec(0, 8, fill_values);
    return values;
}

#include "CuckooHashSetTest.h"
#include "HashGraphV1SetTest.h"
#include "HashGraphV2SetTest.h"

int main(int argc, char* argv[]) {
   ::testing::InitGoogleTest( &argc, argv );
   return RUN_ALL_TESTS();
}
