#pragma once

#include <TNL/Algorithms/ParallelFor.h>
#include <TNL/Containers/Array.h>

#include <memory>


template<typename T>
struct Value {
    __cuda_callable__ Value(const T& key) : key(key) {}
    
    __cuda_callable__ Value<T>& operator=(const Value<T>& other) {
        this->key = other.key;
        return *this;
    }
    
    template<class Device>
    static TNL::Containers::Array<Value<T>, Device> fill(const Array<T, Device>& values) {
        TNL::Containers::Array<Value<T>, Device> arr;
        arr.setSize(values.getSize());

        //for (int i = 0; i < values.getSize(); i++)
        auto aView = arr.getView();
        auto vView = values.getConstView();
        auto _fill = [aView, vView] __cuda_callable__ (int i) mutable {
            aView[i] = Value(vView[i]);
        };
        TNL::Algorithms::ParallelFor<Device>::exec(0, values.getSize(), _fill);
        return std::move(arr);
    }
    
    T key;
};
